@@ -10,11 +10,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import six
+import functools
 
 from oslo_db import exception as db_exc
 from oslo_utils import excutils
-from sqlalchemy.ext import associationproxy
+import sqlalchemy
+from sqlalchemy.ext.associationproxy import ASSOCIATION_PROXY
 from sqlalchemy.orm import exc
 from sqlalchemy.orm import properties
 
@@ -37,11 +38,12 @@ def get_and_validate_sort_keys(sorts, model):
     for sort_key in sort_keys:
         try:
             sort_key_attr = getattr(model, sort_key)
-        except AttributeError:
+        except AttributeError as e:
             # Extension attributes don't support sorting. Because it
             # existed in attr_info, it will be caught here.
             msg = _("'%s' is an invalid attribute for sort key") % sort_key
-            raise n_exc.BadRequest(resource=model.__tablename__, msg=msg)
+            raise n_exc.BadRequest(
+                resource=model.__tablename__, msg=msg) from e
         if isinstance(sort_key_attr.property,
                       properties.RelationshipProperty):
             msg = _("Attribute '%(attr)s' references another resource and "
@@ -96,7 +98,7 @@ def reraise_as_retryrequest(function):
         Exception's as a RetryRequest.
     :raises RetryRequest: If the wrapped function raises retriable exception.
     """
-    @six.wraps(function)
+    @functools.wraps(function)
     def _wrapped(*args, **kwargs):
         try:
             return function(*args, **kwargs)
@@ -148,11 +150,12 @@ def filter_non_model_columns(data, model):
     :returns: A new dict who's keys are columns in model or are association
         proxies of the model.
     """
-    columns = [c.name for c in model.__table__.columns]
-    return dict((k, v) for (k, v) in
-                data.items() if k in columns or
-                isinstance(getattr(model, k, None),
-                           associationproxy.AssociationProxy))
+    mapper = sqlalchemy.inspect(model)
+    columns = set(c.name for c in mapper.columns)
+    columns.update(d.value_attr for d in mapper.all_orm_descriptors
+                   if d.extension_type is ASSOCIATION_PROXY)
+    return dict((k, v) for (k, v)
+                in data.items() if k in columns)
 
 
 def model_query_scope_is_project(context, model):
@@ -163,7 +166,32 @@ def model_query_scope_is_project(context, model):
     :returns: True if the context is not admin and not advsvc and the model
         has a project_id. False otherwise.
     """
-    # Unless a context has 'admin' or 'advanced-service' rights the
+    if not hasattr(model, 'project_id'):
+        # If model doesn't have project_id, there is no need to scope query to
+        # just one project
+        return False
+    if context.is_advsvc:
+        # For context which has 'advanced-service' rights the
+        # query will not be scoped to a single project_id
+        return False
+    # Unless context has 'admin' rights the
     # query will be scoped to a single project_id
-    return ((not context.is_admin and hasattr(model, 'project_id')) and
-            (not context.is_advsvc and hasattr(model, 'project_id')))
+    return not context.is_admin
+
+
+def model_query(context, model):
+    """Query the context for the said model.
+
+    :param context: The context to use for the query.
+    :param model: The model to query for.
+    :returns: A query from the said context for the said model.
+    """
+    query = context.session.query(model)
+    # define basic filter condition for model query
+    query_filter = None
+    if model_query_scope_is_project(context, model):
+        query_filter = (model.tenant_id == context.tenant_id)
+
+    if query_filter is not None:
+        query = query.filter(query_filter)
+    return query
